FILE_TYPE = 'file_type'
FILE_MASK = 'file_mask'
DELIMITED = 'DELIMITED'
FIXED = 'FIXED'
XML = 'XML'
DELIMITER = 'delimiter'
HAS_HEADER = 'has_header'
FIELD_DEFINITIONS = 'field_definitions'
FIELD_NAME = 'field_name'
START_POSITION = 'start_position'
FIELD_LENGTH = 'field_length'
RECORD_ELEMENT_NAME = 'record_element_name'
XML_NODE_NAME = 'xml_node_name'
INPUT_DIRECTORY = 'input_directory'
COMPLETED_DIRECTORY = 'completed_directory'
POSITION_IN_ROW = 'position_in_row'



import json
from dataclasses import dataclass, field
from typing import List

import file_ripper.fileconstants as fc


@dataclass
class FieldDefinition:
    field_name: str
    file_type: str
    start_position: int
    field_length: int
    xml_node_name: str
    position_in_row: int

    def __init__(self, field_name: str, file_type: str, start_position: int = None,
                 field_length: int = None, xml_node_name: str = '', position_in_row: int = None):
        self._validate(file_type, field_name, start_position, field_length, position_in_row)
        self.file_type = file_type
        self.field_name = field_name
        self.start_position = start_position
        self.field_length = field_length
        self.position_in_row = position_in_row
        self.xml_node_name = xml_node_name if xml_node_name else field_name

    @classmethod
    def create_from_dict(cls, file_type, field_definition: dict):
        return cls(file_type=file_type, **field_definition)

    @staticmethod
    def _validate(file_type, field_name, start_position, field_length, position_in_row):
        if not field_name:
            raise ValueError('field_name is required')

        if not file_type:
            raise ValueError('file_type is required')

        if file_type == fc.FIXED and (start_position is None or field_length is None):
            raise ValueError('start_position and field_length are required for a fixed position field')

        if file_type == fc.DELIMITED and position_in_row is None:
            raise ValueError('position_in_row is required for delimited files')

        # if file_type == fc.XML and not xml_node_name:
        #     raise ValueError('xml_node_name is required for a field in xml')


@dataclass
class FileDefinition:
    file_type: str
    field_definitions: List[FieldDefinition]
    has_header: bool = field(default=False)
    delimiter: str = field(default='')
    record_element_name: str = field(default='')
    input_directory: str = field(default='')
    completed_directory: str = field(default='')
    file_mask: str = field(default='')

    def __init__(self, file_type, field_definitions, has_header=False, delimiter='',
                 record_element_name='', input_directory='', completed_directory='',
                 file_mask=''):
        self._validate(file_type, field_definitions, delimiter, record_element_name)
        self.file_type = file_type
        self.field_definitions = field_definitions
        self.has_header = has_header
        self.delimiter = delimiter
        self.record_element_name = record_element_name
        self.input_directory = input_directory
        self.completed_directory = completed_directory
        self.file_mask = file_mask

    @classmethod
    def create_from_dict(cls, json_data: dict):
        json_copy = json_data.copy()
        json_copy[fc.FIELD_DEFINITIONS] = [FieldDefinition.create_from_dict(json_copy[fc.FILE_TYPE], obj)
                                           for obj in json_copy[fc.FIELD_DEFINITIONS]]
        return cls(**json_copy)

    @staticmethod
    def _validate(file_type, field_definitions, delimiter, record_element_name):
        if not file_type:
            raise ValueError('file_type is required')

        if not field_definitions:
            raise ValueError('field_definitions is required')

        if file_type == fc.DELIMITED and not delimiter:
            raise ValueError('delimiter is required for delimited files')

        if file_type == fc.XML and not record_element_name:
            raise ValueError('record_element_name is required for xml files')



import json
from unittest import TestCase

import file_ripper.fileconstants as fc
from file_ripper.filedefinition import FileDefinition, FieldDefinition


class TestFieldDefinitionConstruction(TestCase):
    def test_field_name_blank(self):
        with self.assertRaises(ValueError):
            FieldDefinition('', fc.DELIMITED)

    def test_file_type_blank(self):
        with self.assertRaises(ValueError):
            FieldDefinition('name', '')

    def test_file_type_delimited_field_name_only(self):
        with self.assertRaises(ValueError):
            FieldDefinition('name', fc.DELIMITED)

    def test_file_type_fixed_without_start_position(self):
        with self.assertRaises(ValueError):
            FieldDefinition('name', fc.FIXED, start_position=None)

    def test_file_type_fixed_without_field_length(self):
        with self.assertRaises(ValueError):
            FieldDefinition('name', fc.FIXED, start_position=0, field_length=None)

    def test_file_type_xml_without_xml_node_name(self):
        field_definition = FieldDefinition('name', fc.XML, xml_node_name='')
        self.assertEqual('name', field_definition.xml_node_name)


class TestFieldDefinitionCreateFromDict(TestCase):
    def setUp(self) -> None:
        self.field_definition = FieldDefinition.create_from_dict(fc.XML, {
            fc.FIELD_NAME: 'age',
            fc.START_POSITION: 10,
            fc.FIELD_LENGTH: 5,
            fc.XML_NODE_NAME: 'personAge',
            fc.POSITION_IN_ROW: 0
        })

    def test_file_type_set(self):
        self.assertEqual('XML', self.field_definition.file_type)

    def test_field_name_set(self):
        self.assertEqual('age', self.field_definition.field_name)

    def test_start_position_set(self):
        self.assertEqual(10, self.field_definition.start_position)

    def test_field_length_set(self):
        self.assertEqual(5, self.field_definition.field_length)

    def test_xml_node_name_set(self):
        self.assertEqual('personAge', self.field_definition.xml_node_name)


class TestFileDefinitionConstruction(TestCase):
    def setUp(self) -> None:
        self.field_definition = FieldDefinition('dob', 'XML')
        self.file_definition = FileDefinition('XML', [self.field_definition], has_header=True,
                                              delimiter='|', record_element_name='person',
                                              input_directory='input', completed_directory='completed',
                                              file_mask='mask')

    def test_file_type_and_field_definitions(self):
        self.assertEqual('XML', self.file_definition.file_type)
        self.assertTrue(len(self.file_definition.field_definitions) == 1)
        self.assertEqual(self.field_definition, self.file_definition.field_definitions[0])

    def test_has_header(self):
        self.assertTrue(self.file_definition.has_header)

    def test_delimiter(self):
        self.assertEqual('|', self.file_definition.delimiter)

    def test_record_element_name(self):
        self.assertEqual('person', self.file_definition.record_element_name)

    def test_input_directory(self):
        self.assertEqual('input', self.file_definition.input_directory)

    def test_completed_directory(self):
        self.assertEqual('completed', self.file_definition.completed_directory)

    def test_file_mask(self):
        self.assertEqual('mask', self.file_definition.file_mask)

    def test_file_type_missing(self):
        with self.assertRaises(ValueError):
            FileDefinition('', [self.file_definition])

    def test_file_definitions_missing(self):
        with self.assertRaises(ValueError):
            FileDefinition('XML', [])

    def test_delimited_file_delimiter_missing(self):
        with self.assertRaises(ValueError):
            FileDefinition(fc.DELIMITED, [self.field_definition], delimiter='')

    def test_xml_file_record_element_name_missing(self):
        with self.assertRaises(ValueError):
            FileDefinition(fc.XML, [self.field_definition], record_element_name='')


class TestFileDefinitionCreateDelimitedFromDict(TestCase):
    def setUp(self) -> None:
        self.file_definition = FileDefinition.create_from_dict({
            fc.FILE_TYPE: fc.DELIMITED,
            fc.DELIMITER: ',',
            fc.RECORD_ELEMENT_NAME: 'record',
            fc.HAS_HEADER: True,
            fc.INPUT_DIRECTORY: 'input',
            fc.COMPLETED_DIRECTORY: 'completed',
            fc.FILE_MASK: 'mask',
            fc.FIELD_DEFINITIONS: [{
                fc.FIELD_NAME: 'name',
                fc.POSITION_IN_ROW: 0
            }]
        })

    def test_file_type(self):
        self.assertEqual(fc.DELIMITED, self.file_definition.file_type)

    def test_delimiter(self):
        self.assertEqual(',', self.file_definition.delimiter)

    def test_record_element_name(self):
        self.assertEqual('record', self.file_definition.record_element_name)

    def test_has_header(self):
        self.assertTrue(self.file_definition.has_header)

    def test_input_directory(self):
        self.assertEqual('input', self.file_definition.input_directory)

    def test_completed_directory(self):
        self.assertEqual('completed', self.file_definition.completed_directory)

    def test_file_mask(self):
        self.assertEqual('mask', self.file_definition.file_mask)

    def test_field_definitions(self):
        self.assertTrue(len(self.file_definition.field_definitions) == 1)



from typing import IO, Callable, Dict
from xml.etree.ElementTree import fromstring

import file_ripper.fileconstants as fc
from file_ripper.filedefinition import FileDefinition
from file_ripper.fileinstance import FileInstance, FileRow


class FileService:
    def __init__(self, file_definition):
        self.file_definition = file_definition

    def process(self, file: IO) -> FileInstance:
        records = self.process_file_records(file)
        return FileInstance(file.name, records)

    def process_file_records(self, file):
        raise NotImplementedError('Please use a valid implementation of FileService to read files')


class XmlFileService(FileService):
    def __init__(self, file_definition):
        super().__init__(file_definition)

    def process_file_records(self, file: IO):
        tree = fromstring(''.join(file.readlines()))

        file_rows = []
        for item in tree.findall(f'./{self.file_definition.record_element_name}'):
            record = {}
            for field_def in self.file_definition.field_definitions:
                record[field_def.field_name] = item.find(f'{field_def.field_name}').text
            file_rows.append(FileRow(record))

        return file_rows


class FlatFileService(FileService):
    def __init__(self, file_definition: FileDefinition):
        super().__init__(file_definition)

    def process_file_records(self, file: IO):
        records = []
        lines = file.readlines()
        if self.file_definition.has_header:
            lines.pop(0)

        for line in lines:
            record = self.process_delimited_line(line) \
                if self.file_definition.file_type == fc.DELIMITED \
                else self.process_fixed_line(line)
            records.append(record)

        return records

    def process_delimited_line(self, line):
        fields = [field.rstrip() for field in line.split(self.file_definition.delimiter)]
        record = {}

        field_count = len(self.file_definition.field_definitions)
        if field_count != len(fields):
            raise OSError('File records do not match file definition')

        for i in range(0, field_count):
            record[self.file_definition.field_definitions[i].field_name] = \
                fields[self.file_definition.field_definitions[i].position_in_row]
        return FileRow(record)

    def process_fixed_line(self, line):
        record = {}
        for field_def in self.file_definition.field_definitions:
            end_position = field_def.start_position + field_def.field_length
            if end_position > len(line.rstrip()):
                raise IndexError(f'field {field_def.field_name} extends past the end of line')
            record[field_def.field_name] = line[field_def.start_position:end_position].rstrip()
        return FileRow(record)


def create_file_service(file_definition):
    if file_definition.file_type == fc.XML:
        return XmlFileService(file_definition)
    elif file_definition.file_type == fc.DELIMITED:
        return FlatFileService(file_definition)
    elif file_definition.file_type == fc.FIXED:
        return FlatFileService(file_definition)
    else:
        raise ValueError(f'file_definition is configured for unsupported file_type: {file_definition.file_type}')



import os
import unittest
from unittest import TestCase

import file_ripper.fileconstants as fc
from file_ripper.filedefinition import FileDefinition, FieldDefinition
from file_ripper.fileservice import XmlFileService, FileService, FlatFileService, create_file_service


class CreateFileServiceTests(TestCase):
    """Test cases for file service factory function code"""
    def test_xml_file_definition(self):
        """Create xml file service given and xml file definition"""
        file_definition = self.create_file_definition(fc.XML, record_element_name='record')
        file_service = create_file_service(file_definition)
        self.assertTrue(isinstance(file_service, XmlFileService))

    def test_delimited_file_definition(self):
        """Create delimited file service given a delimited file definition"""
        file_definition = self.create_file_definition(fc.DELIMITED, delimiter='\t', position_in_row=0)
        file_service = create_file_service(file_definition)
        self.assertTrue(isinstance(file_service, FlatFileService))

    def test_fixed_file_definition(self):
        """Create fixed file service given fixed file definition"""
        file_definition = self.create_file_definition(fc.FIXED, start_position=0, field_length=12)
        file_service = create_file_service(file_definition)
        self.assertTrue(isinstance(file_service, FlatFileService))

    def test_invalid_file_type(self):
        """Raise value error when invalid file type is created"""
        file_definition = self.create_file_definition('file_type')
        with self.assertRaises(ValueError):
            create_file_service(file_definition)

    @staticmethod
    def create_file_definition(file_type, delimiter=None, record_element_name=None, start_position=None,
                               field_length=None, position_in_row=None):
        field_definitions = [FieldDefinition('name', file_type, start_position=start_position,
                                             field_length=field_length, position_in_row=position_in_row)]
        return FileDefinition(file_type, field_definitions,
                              record_element_name=record_element_name,
                              delimiter=delimiter)


class FileServiceTests(unittest.TestCase):
    def create_file_definitions(self, file_type):
        self.file_name = None
        self.field_definitions = [
            FieldDefinition('name', file_type, 0, 13, position_in_row=0),
            FieldDefinition('age', file_type, 13, 9, position_in_row=2),
            FieldDefinition('dob', file_type, 22, 10, position_in_row=1)
        ]

    def assert_valid_file_output(self, output_file_name, file_records):
        self.assertEqual(self.file_name, output_file_name)
        self.assertTrue(isinstance(file_records, list))
        self.assertEqual(4, len(file_records))
        self.assert_valid_records(file_records)

    def assert_valid_records(self, file_records):
        self.assertEqual('Aaron', file_records[0]['name'])
        self.assertEqual('39', file_records[0]['age'])
        self.assertEqual('09/04/1980', file_records[0]['dob'])
        self.assertEqual('Gene', file_records[1]['name'])
        self.assertEqual('61', file_records[1]['age'])
        self.assertEqual('01/15/1958', file_records[1]['dob'])
        self.assertEqual('Xander', file_records[2]['name'])
        self.assertEqual('4', file_records[2]['age'])
        self.assertEqual('11/22/2014', file_records[2]['dob'])

    def test_process_records_not_implemented(self):
        file_service = FileService(FileDefinition(fc.FIXED, [FieldDefinition('hello', 'file_type')]))
        with self.assertRaises(NotImplementedError):
            file_service.process_file_records(None)

    def tearDown(self):
        try:
            if self.file_name:
                os.remove(self.file_name)
        except Exception:
            pass


class DelimitedFileServiceTests(FileServiceTests):
    def setUp(self):
        super().create_file_definitions(fc.DELIMITED)
        self.file_definition = FileDefinition(fc.DELIMITED, self.field_definitions, True, '|')
        self.file_service = FlatFileService(self.file_definition)
        self.file_name = 'Valid-delimited-09032019.txt'
        with open(self.file_name, 'w') as f:
            f.write('Name|DOB|Age\n')
            f.write('Aaron|09/04/1980|39\n')
            f.write('Gene|01/15/1958|61\n')
            f.write('Xander|11/22/2014|4\n')
            f.write('Mason|04/13/2007|12\n')

    def test_process_given_pipe_delimiter(self):
        with open(self.file_name, 'r') as file:
            file_instance = self.file_service.process(file)
            self.assert_valid_file_output(file_instance.file_name, file_instance.file_rows)

    def test_process_given_invalid_file(self):
        with open(self.file_name, 'r') as file:
            self.file_definition.field_definitions.remove(self.file_definition.field_definitions[-1])
            with self.assertRaises(OSError):
                self.file_service.process(file)


class FixedFileServiceTests(FileServiceTests):
    def setUp(self):
        super(FixedFileServiceTests, self).create_file_definitions(fc.FIXED)
        self.file_definition = FileDefinition(fc.FIXED, self.field_definitions, True)
        self.file_service = create_file_service(self.file_definition)
        self.file_name = 'Valid-fixed-09032019.txt'
        with open(self.file_name, 'w') as f:
            f.write('Name         Age      DOB       \n')
            f.write('Aaron        39       09/04/1980\n')
            f.write('Gene         61       01/15/1958\n')
            f.write('Xander       4        11/22/2014\n')
            f.write('Mason        12       04/13/2007\n')

    def test_process(self):
        with open(self.file_name, 'r') as file:
            file_instance = self.file_service.process(file)
            self.assert_valid_file_output(file_instance.file_name, file_instance.file_rows)

    def test_process_invalid_file_records_too_short(self):
        with open(self.file_name, 'r') as file:
            self.file_definition.field_definitions.append(
                FieldDefinition.create_from_dict(fc.FIXED, {
                    fc.FIELD_NAME: 'address',
                    fc.START_POSITION: 32,
                    fc.FIELD_LENGTH: 2
                }))
            self.assertRaises(IndexError, self.file_service.process, file)


class XmlFileServiceTests(FileServiceTests):
    def setUp(self):
        super(XmlFileServiceTests, self).create_file_definitions(fc.XML)
        self.file_definition = FileDefinition(fc.XML, self.field_definitions, record_element_name='person')
        self.file_service = XmlFileService(self.file_definition)
        self.file_name = 'Valid-xml-09032019.txt'
        with open(self.file_name, 'w') as f:
            self.file = f
            f.write('<people>\n')
            self.write_xml_record('Aaron', 39, '09/04/1980')
            self.write_xml_record('Gene', 61, '01/15/1958')
            self.write_xml_record('Xander', 4, '11/22/2014')
            self.write_xml_record('Mason', 12, '04/13/2007')
            f.write('</people>\n')

    def write_xml_record(self, name, age, dob):
        self.file.write('\t<person>\n')
        self.file.write(f'\t\t<name>{name}</name>\n')
        self.file.write(f'\t\t<age>{age}</age>\n')
        self.file.write(f'\t\t<dob>{dob}</dob>\n')
        self.file.write('\t</person>\n')

    def test_process(self):
        with open(self.file_name, 'r') as file:
            file_instance = self.file_service.process(file)
            self.assert_valid_file_output(file_instance.file_name, file_instance.file_rows)

    def test_process_given_invalid_file_missing_attribute(self):
        with open(self.file_name, 'r') as file:
            self.file_definition.field_definitions.append(
                FieldDefinition.create_from_dict(fc.XML, {fc.FIELD_NAME: 'address'}))
            self.assertRaises(AttributeError, self.file_service.process, file)



from unittest import TestCase

from file_ripper.fileinstance import FileRow, FileInstance


class TestFileRowConstruction(TestCase):
    def test_empty_dict(self):
        file_row = FileRow({})
        self.assertTrue(len(file_row.fields) == 0)

    def test_non_empty_dict(self):
        file_row = FileRow({'key': 'value'})
        self.assertTrue(len(file_row.fields) == 1)
        self.assertEqual('value', file_row.fields['key'])

    def test_none(self):
        file_row = FileRow(None)
        self.assertTrue(file_row.fields is None)


class TestFileRowContainerProtocol(TestCase):
    def setUp(self) -> None:
        self.file_row = FileRow({'name': 'Aaron', 'age': '26'})

    def test_positive_contained(self):
        self.assertTrue('name' in self.file_row)

    def test_negative_contained(self):
        self.assertFalse('dob' in self.file_row)

    def test_positive_not_contained(self):
        self.assertTrue('addr' not in self.file_row)

    def test_negative_not_contained(self):
        self.assertFalse('age' not in self.file_row)


class TestFileRowSizedProtocol(TestCase):
    def test_zero(self):
        row = FileRow({})
        self.assertEqual(0, len(row))

    def test_one(self):
        row = FileRow({'key': 'value'})
        self.assertEqual(1, len(row))

    def test_five(self):
        row = FileRow({'key1': 'value', 'key2': 'value', 'key3': 'value', 'key4': 'value', 'key5': 'value'})
        self.assertEqual(5, len(row))


class TestFileRowIterableProtocol(TestCase):
    def setUp(self) -> None:
        self.row = FileRow({'name': 'Xander', 'age': '6', 'dob': '11/22/63'})

    def test_iter(self):
        i = iter(self.row)
        self.assertEqual('name', next(i))
        self.assertEqual('age', next(i))
        self.assertEqual('dob', next(i))
        with self.assertRaises(StopIteration):
            next(i)

    def test_for_loop(self):
        expected = ['name', 'age', 'dob']
        index = 0
        for i in self.row:
            self.assertEqual(expected[index], i)
            index += 1


class TestFileRowSequenceProtocol(TestCase):
    def setUp(self) -> None:
        self.row = FileRow({'name': 'Xander', 'age': '6', 'addr': '123 Main St', 'dob': '11/22/63'})

    def test_indexing(self):
        self.assertEqual('Xander', self.row['name'])
        self.assertEqual('6', self.row['age'])
        self.assertEqual('123 Main St', self.row['addr'])
        self.assertEqual('11/22/63', self.row['dob'])

    def test_slicing(self):
        with self.assertRaises(TypeError):
            fields = self.row['name':'age']

    def test_reversing(self):
        with self.assertRaises(TypeError):
            reversed(self.row)


class TestFileInstanceConstruction(TestCase):
    def test_file_name_and_file_rows(self):
        file_row = FileRow({'name': 'John'})
        file_instance = FileInstance('file_name', [file_row])
        self.assertEqual('file_name', file_instance.file_name)
        self.assertTrue(len(file_instance.file_rows) == 1)
        self.assertEqual([file_row], file_instance.file_rows)

    def test_file_name_missing(self):
        with self.assertRaises(TypeError):
            FileInstance(file_rows=[])

    def test_file_rows_missing(self):
        with self.assertRaises(TypeError):
            FileInstance(file_name='file_name')


class TestFileInstanceContainerProtocol(TestCase):
    def setUp(self) -> None:
        self.file_row = FileRow({'name': 'Alex', 'age': '27'})
        self.file_instance = FileInstance('file_name', [self.file_row])

    def test_positive_contains(self):
        self.assertTrue(self.file_row in self.file_instance)

    def test_negative_contains(self):
        self.assertFalse(FileRow({'name': 'Jeff'}) in self.file_instance)

    def test_negative_not_contains(self):
        self.assertFalse(self.file_row not in self.file_instance)

    def test_positive_not_contains(self):
        self.assertTrue(FileRow({'name': 'Jeff'}) not in self.file_instance)


class TestFileInstanceSizedProtocol(TestCase):
    def test_zero(self):
        self.assertTrue(len(FileInstance('', [])) == 0)

    def test_one(self):
        file_row = FileRow({})
        self.assertTrue(len(FileInstance('', [file_row])) == 1)

    def test_five(self):
        file_rows = [FileRow({}), FileRow({}), FileRow({}), FileRow({}), FileRow({})]
        self.assertTrue(len(FileInstance('', file_rows)) == 5)


class TestFileInstanceIterableProtocol(TestCase):
    def setUp(self) -> None:
        self.file_rows = [FileRow({'name': 'Aaron'}), FileRow({'age': '40'}), FileRow({'dob': '06/12/1980'})]
        self.file_instance = FileInstance('file_name', self.file_rows)

    def test_iter(self):
        i = iter(self.file_instance)
        self.assertEqual(self.file_rows[0], next(i))
        self.assertEqual(self.file_rows[1], next(i))
        self.assertEqual(self.file_rows[2], next(i))
        with self.assertRaises(StopIteration):
            next(i)

    def test_for_loop(self):
        index = 0
        for row in self.file_instance:
            self.assertEqual(self.file_rows[index], row)
            index += 1


class TestFileInstanceSequenceProtocol(TestCase):
    def setUp(self) -> None:
        self.file_rows = [FileRow({'name': 'John'}), FileRow({'name': 'Joe'}), FileRow({'name': 'Jeff'}),
                          FileRow({'name': 'Adam'}), FileRow({'name': 'Alex'}), FileRow({'name': 'Steve'})]
        self.file_instance = FileInstance('', self.file_rows)

    def test_indexing(self):
        self.assertEqual(self.file_rows[1], self.file_instance[1])

    def test_slicing_beginning(self):
        self.assertEqual(self.file_rows[:2], self.file_instance[:2])

    def test_slicing_end(self):
        self.assertEqual(self.file_rows[4:], self.file_instance[4:])

    def test_slicing_middle(self):
        self.assertEqual(self.file_rows[2:4], self.file_instance[2:4])

    def test_reversed(self):
        expected = reversed(self.file_rows)
        rev = reversed(self.file_instance)
        self.assertEqual(next(expected), next(rev))
        self.assertEqual(next(expected), next(rev))
        self.assertEqual(next(expected), next(rev))
        self.assertEqual(next(expected), next(rev))
        self.assertEqual(next(expected), next(rev))
        self.assertEqual(next(expected), next(rev))
        with self.assertRaises(StopIteration):
            next(rev)


class TestFileInstanceEquality(TestCase):
    def setUp(self) -> None:
        self.file_name = 'file_name'
        self.file_rows = [FileRow({'name': 'Dominic'}), FileRow({'age': '44', 'dob': '11/11/1977'})]

    def test_are_equal(self):
        self.assertTrue(FileInstance(self.file_name, self.file_rows) == FileInstance(self.file_name, self.file_rows))

    def test_different_file_names(self):
        self.assertFalse(FileInstance(self.file_name + '1', self.file_rows) == FileInstance(self.file_name + '2', self.file_rows))

    def test_different_file_rows(self):
        self.assertFalse(FileInstance(self.file_name, self.file_rows) == FileInstance(self.file_name, [FileRow({})]))



from dataclasses import dataclass
from typing import List, Dict


@dataclass
class FileRow:
    fields: Dict[str, str]

    def __contains__(self, item):
        return item in self.fields

    def __len__(self):
        return len(self.fields)

    def __iter__(self):
        return iter(self.fields)

    def __getitem__(self, item):
        return self.fields[item]

    def __reversed__(self):
        return reversed(self.fields)


@dataclass
class FileInstance:
    file_name: str
    file_rows: List[FileRow]

    def __contains__(self, item):
        return item in self.file_rows

    def __len__(self):
        return len(self.file_rows)

    def __iter__(self):
        return iter(self.file_rows)

    def __getitem__(self, item):
        return self.file_rows[item]



import glob
import os
from typing import IO, List

from file_ripper.filedefinition import FileDefinition
from file_ripper.fileinstance import FileInstance
from file_ripper.fileservice import create_file_service


def validate_file_definition(file_definition: FileDefinition):
    if not file_definition.input_directory:
        raise ValueError('input_directory is required')

    if not file_definition.file_mask:
        raise ValueError('file_mask is required')

    if file_definition.completed_directory and not os.path.exists(file_definition.completed_directory):
        os.mkdir(file_definition.completed_directory)


def move_file_if_needed(file_name: str, file_definition: FileDefinition):
    if file_definition.completed_directory:
        os.rename(f'{file_definition.input_directory}/{os.path.split(file_name)[-1]}',
                  f'{file_definition.completed_directory}/{os.path.split(file_name)[-1]}')


def rip_file(file: IO, file_definition: FileDefinition) -> FileInstance:
    file_service = create_file_service(file_definition)
    return file_service.process(file)


def rip_files(files: List[IO], file_definition: FileDefinition) -> List[FileInstance]:
    return [rip_file(f, file_definition) for f in files]


def find_and_rip_files(file_definition: FileDefinition) -> List[FileInstance]:
    validate_file_definition(file_definition)
    file_output_list = []

    for file_name in glob.glob(f'{file_definition.input_directory}/{file_definition.file_mask}'):
        with open(file_name, 'r') as file:
            file_output_list.append(rip_file(file, file_definition))

        move_file_if_needed(file_name, file_definition)

    return file_output_list



from file_ripper.fileripper import rip_file, rip_files, find_and_rip_files
from file_ripper.filedefinition import FileDefinition, FieldDefinition
from file_ripper.fileinstance import FileInstance
import file_ripper.fileconstants as file_constants



from unittest import TestCase

import file_ripper.fileconstants as fc


class FileConstantsTests(TestCase):
    def test_file_type(self):
        self.assertEqual(fc.FILE_TYPE, 'file_type')

    def test_file_mask(self):
        self.assertEqual(fc.FILE_MASK, 'file_mask')

    def test_delimited(self):
        self.assertEqual(fc.DELIMITED, 'DELIMITED')

    def test_fixed(self):
        self.assertEqual(fc.FIXED, 'FIXED')

    def test_xml(self):
        self.assertEqual(fc.XML, 'XML')

    def test_delimiter(self):
        self.assertEqual(fc.DELIMITER, 'delimiter')

    def test_has_header(self):
        self.assertEqual(fc.HAS_HEADER, 'has_header')

    def test_field_definitions(self):
        self.assertEqual(fc.FIELD_DEFINITIONS, 'field_definitions')

    def test_field_name(self):
        self.assertEqual(fc.FIELD_NAME, 'field_name')

    def test_start_position(self):
        self.assertEqual(fc.START_POSITION, 'start_position')

    def test_field_length(self):
        self.assertEqual(fc.FIELD_LENGTH, 'field_length')

    def test_record_element_name(self):
        self.assertEqual(fc.RECORD_ELEMENT_NAME, 'record_element_name')

    def test_xml_node_name(self):
        self.assertEqual(fc.XML_NODE_NAME, 'xml_node_name')

    def test_input_directory(self):
        self.assertEqual(fc.INPUT_DIRECTORY, 'input_directory')

    def test_completed_directory(self):
        self.assertEqual(fc.COMPLETED_DIRECTORY, 'completed_directory')

    def test_position_in_row(self):
        self.assertEqual('position_in_row', fc.POSITION_IN_ROW)


import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()
    setuptools.setup(
        name='file-ripper',
        version='1.3.1',
        author='Aaron Smith',
        author_email='asmitty92@gmail.com',
        license='MIT',
        long_description=long_description,
        long_description_content_type='text/markdown',
        packages=setuptools.find_packages(),
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',

        ]
    )




import logging
from os import path, mkdir
from logging.handlers import TimedRotatingFileHandler


log_directory = '/Users/asmitty/workspace/python/file_ripper_py/file_ripper/logs'
log_file_name = 'file_ripper_logs.txt'


def create_file_ripper_logger():
    formatter = logging.Formatter(
        '%(asctime)s %(name)-8s %(levelname)-8s %(message)s')

    logger = logging.getLogger()
    logger.addHandler(create_stream_handler(formatter))
    logger.addHandler(create_file_handler(formatter))
    logger.setLevel(logging.INFO)
    return logger


def create_file_handler(formatter):
    if not path.exists(log_directory):
        mkdir(log_directory)
    handler = TimedRotatingFileHandler(path.join(log_directory, log_file_name), encoding='UTF-8')
    handler.setFormatter(formatter)
    return handler


def create_stream_handler(formatter):
    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    return handler
